@@ -19,23 +19,30 @@ commit-notify     Start the graphical notifier of commits.
 gannotate         GTK+ annotate. 
 gbranch           GTK+ branching. 
 gcheckout         GTK+ checkout. 
-gcommit           GTK+ commit dialog 
+gcommit           GTK+ commit dialog.
 gconflicts        GTK+ conflicts. 
 gdiff             Show differences in working tree in a GTK+ Window. 
 ginit             Initialise a new branch.
 gmissing          GTK+ missing revisions dialog. 
 gpreferences      GTK+ preferences dialog. 
-gpush             GTK+ push. 
-gstatus           GTK+ status dialog 
+gpush             GTK+ push.
+gsend             GTK+ send merge directive.
+gstatus           GTK+ status dialog.
 gtags             Manage branch tags.
 visualise         Graphically visualise this branch. 
 """
 
+import sys
+
 import bzrlib
 
-__version__ = '0.90.0'
-version_info = tuple(int(n) for n in __version__.split('.'))
+version_info = (0, 93, 0, 'final', 0)
 
+if version_info[3] == 'final':
+    version_string = '%d.%d.%d' % version_info[:3]
+else:
+    version_string = '%d.%d.%d%s%d' % version_info
+__version__ = version_string
 
 def check_bzrlib_version(desired):
     """Check that bzrlib is compatible.
@@ -59,14 +66,15 @@ def check_bzrlib_version(desired):
         from bzrlib.errors import BzrError
         warning('Installed Bazaar version %s is too old to be used with bzr-gtk'
                 ' %s.' % (bzrlib.__version__, __version__))
-        raise BzrError('Version mismatch: %r' % (version_info,) )
+        raise BzrError('Version mismatch: %r, %r' % (version_info, bzrlib.version_info) )
     else:
         warning('bzr-gtk is not up to date with installed bzr version %s.'
                 ' \nThere should be a newer version available, e.g. %i.%i.' 
                 % (bzrlib.__version__, bzrlib_version[0], bzrlib_version[1]))
 
 
-check_bzrlib_version(version_info[:2])
+if version_info[2] == "final":
+    check_bzrlib_version(version_info[:2])
 
 from bzrlib.trace import warning
 if __name__ != 'bzrlib.plugins.gtk':
@@ -220,12 +228,7 @@ def start_viz_window(branch, revision, limit=None):
     :return: The viz window object.
     """
     from viz.branchwin import BranchWindow
-    branch.lock_read()
-    pp = BranchWindow()
-    pp.set_branch(branch, revision, limit)
-    # cleanup locks when the window is closed
-    pp.connect("destroy", lambda w: branch.unlock())
-    return pp
+    return BranchWindow(branch, revision, limit)
 
 
 class cmd_visualise(Command):
@@ -247,22 +250,18 @@ class cmd_visualise(Command):
     def run(self, location=".", revision=None, limit=None):
         set_ui_factory()
         (br, path) = branch.Branch.open_containing(location)
-        br.lock_read()
-        try:
-            if revision is None:
-                revid = br.last_revision()
-                if revid is None:
-                    return
-            else:
-                (revno, revid) = revision[0].in_history(br)
+        if revision is None:
+            revid = br.last_revision()
+            if revid is None:
+                return
+        else:
+            (revno, revid) = revision[0].in_history(br)
 
-            import gtk
-            pp = start_viz_window(br, revid, limit)
-            pp.connect("destroy", lambda w: gtk.main_quit())
-            pp.show()
-            gtk.main()
-        finally:
-            br.unlock()
+        import gtk
+        pp = start_viz_window(br, revid, limit)
+        pp.connect("destroy", lambda w: gtk.main_quit())
+        pp.show()
+        gtk.main()
 
 
 class cmd_gannotate(GTKCommand):
@@ -335,7 +334,7 @@ class cmd_gcommit(GTKCommand):
     """GTK+ commit dialog
 
     Graphical user interface for committing revisions"""
-    
+
     aliases = [ "gci" ]
     takes_args = []
     takes_options = []
@@ -354,12 +353,21 @@ class cmd_gcommit(GTKCommand):
             (wt, path) = workingtree.WorkingTree.open_containing(filename)
             br = wt.branch
         except NoWorkingTree, e:
-            path = e.base
-            (br, path) = branch.Branch.open_containing(path)
-
-        commit = CommitDialog(wt, path, not br)
-        commit.run()
+            from dialog import error_dialog
+            error_dialog(_('Directory does not have a working tree'),
+                         _('Operation aborted.'))
+            return 1 # should this be retval=3?
 
+        # It is a good habit to keep things locked for the duration, but it
+        # could cause difficulties if someone wants to do things in another
+        # window... We could lock_read() until we actually go to commit
+        # changes... Just a thought.
+        wt.lock_write()
+        try:
+            dlg = CommitDialog(wt)
+            return dlg.run()
+        finally:
+            wt.unlock()
 
 
 class cmd_gstatus(GTKCommand):
@@ -382,9 +390,28 @@ class cmd_gstatus(GTKCommand):
         status.run()
 
 
+class cmd_gsend(GTKCommand):
+    """GTK+ send merge directive.
+
+    """
+    def run(self):
+        (br, path) = branch.Branch.open_containing(".")
+        gtk = self.open_display()
+        from bzrlib.plugins.gtk.mergedirective import SendMergeDirectiveDialog
+        from StringIO import StringIO
+        dialog = SendMergeDirectiveDialog(br)
+        if dialog.run() == gtk.RESPONSE_OK:
+            outf = StringIO()
+            outf.writelines(dialog.get_merge_directive().to_lines())
+            mail_client = br.get_config().get_mail_client()
+            mail_client.compose_merge_request(dialog.get_mail_to(), "[MERGE]", 
+                outf.getvalue())
+
+            
+
 
 class cmd_gconflicts(GTKCommand):
-    """ GTK+ conflicts.
+    """GTK+ conflicts.
     
     Select files from the list of conflicts and run an external utility to
     resolve them.
@@ -397,7 +424,6 @@ class cmd_gconflicts(GTKCommand):
         dialog.run()
 
 
-
 class cmd_gpreferences(GTKCommand):
     """ GTK+ preferences dialog.
 
@@ -409,7 +435,6 @@ class cmd_gpreferences(GTKCommand):
         dialog.run()
 
 
-
 class cmd_gmissing(Command):
     """ GTK+ missing revisions dialog.
 
@@ -466,19 +491,20 @@ class cmd_gtags(GTKCommand):
 
 
 commands = [
+    cmd_gannotate, 
+    cmd_gbranch,
+    cmd_gcheckout, 
+    cmd_gcommit, 
+    cmd_gconflicts, 
+    cmd_gdiff,
+    cmd_ginit,
     cmd_gmissing, 
     cmd_gpreferences, 
-    cmd_gconflicts, 
-    cmd_gstatus,
-    cmd_gcommit, 
-    cmd_gannotate, 
-    cmd_visualise, 
-    cmd_gdiff,
     cmd_gpush, 
-    cmd_gcheckout, 
-    cmd_gbranch,
-    cmd_ginit,
-    cmd_gtags
+    cmd_gsend,
+    cmd_gstatus,
+    cmd_gtags,
+    cmd_visualise
     ]
 
 for cmd in commands:
@@ -593,6 +619,68 @@ class cmd_gselftest(GTKCommand):
 register_command(cmd_gselftest)
 
 
+class cmd_test_gtk(GTKCommand):
+    """Version of selftest that just runs the gtk test suite."""
+
+    takes_options = ['verbose',
+                     Option('one', short_name='1',
+                            help='Stop when one test fails.'),
+                     Option('benchmark', help='Run the benchmarks.'),
+                     Option('lsprof-timed',
+                     help='Generate lsprof output for benchmarked'
+                          ' sections of code.'),
+                     Option('list-only',
+                     help='List the tests instead of running them.'),
+                     Option('randomize', type=str, argname="SEED",
+                     help='Randomize the order of tests using the given'
+                          ' seed or "now" for the current time.'),
+                    ]
+    takes_args = ['testspecs*']
+
+    def run(self, verbose=None, one=False, benchmark=None,
+            lsprof_timed=None, list_only=False, randomize=None,
+            testspecs_list=None):
+        from bzrlib import __path__ as bzrlib_path
+        from bzrlib.tests import selftest
+
+        print '%10s: %s' % ('bzrlib', bzrlib_path[0])
+        if benchmark:
+            print 'No benchmarks yet'
+            return 3
+
+            test_suite_factory = bench_suite
+            if verbose is None:
+                verbose = True
+            # TODO: should possibly lock the history file...
+            benchfile = open(".perf_history", "at", buffering=1)
+        else:
+            test_suite_factory = test_suite
+            if verbose is None:
+                verbose = False
+            benchfile = None
+
+        if testspecs_list is not None:
+            pattern = '|'.join(testspecs_list)
+        else:
+            pattern = ".*"
+
+        try:
+            result = selftest(verbose=verbose,
+                              pattern=pattern,
+                              stop_on_failure=one,
+                              test_suite_factory=test_suite_factory,
+                              lsprof_timed=lsprof_timed,
+                              bench_history=benchfile,
+                              list_only=list_only,
+                              random_seed=randomize,
+                             )
+        finally:
+            if benchfile is not None:
+                benchfile.close()
+
+register_command(cmd_test_gtk)
+
+
 import gettext
 gettext.install('olive-gtk')
 
@@ -611,6 +699,10 @@ def test_suite():
     default_encoding = sys.getdefaultencoding()
     try:
         result = TestSuite()
+        try:
+            import_pygtk()
+        except errors.BzrCommandError:
+            return result
         result.addTest(tests.test_suite())
     finally:
         if sys.getdefaultencoding() != default_encoding:
