@@ -28,7 +28,8 @@ from bzrlib.errors import NoSuchRevision
 from bzrlib.revision import NULL_REVISION, CURRENT_REVISION
 
 from colormap import AnnotateColorMap, AnnotateColorSaturation
-from bzrlib.plugins.gtk.logview import LogView
+from bzrlib.plugins.gtk.revisionview import RevisionView
+from bzrlib.plugins.gtk.window import Window
 
 
 (
@@ -41,14 +42,14 @@ from bzrlib.plugins.gtk.logview import LogView
 ) = range(6)
 
 
-class GAnnotateWindow(gtk.Window):
+class GAnnotateWindow(Window):
     """Annotate window."""
 
-    def __init__(self, all=False, plain=False):
+    def __init__(self, all=False, plain=False, parent=None):
         self.all = all
         self.plain = plain
         
-        gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
+        Window.__init__(self, parent)
         
         self.set_icon(self.render_icon(gtk.STOCK_FIND, gtk.ICON_SIZE_BUTTON))
         self.annotate_colormap = AnnotateColorSaturation()
@@ -63,6 +64,7 @@ class GAnnotateWindow(gtk.Window):
         self.branch = branch
         self.tree = tree
         self.file_id = file_id
+        self.revisionview.set_file_id(file_id)
         self.revision_id = getattr(tree, 'get_revision_id', 
                                    lambda: CURRENT_REVISION)()
         
@@ -81,18 +83,17 @@ class GAnnotateWindow(gtk.Window):
             for line_no, (revision, revno, line)\
                     in enumerate(self._annotate(tree, file_id)):
                 if revision.revision_id == last_seen and not self.all:
-                    revno = committer = ""
+                    revno = author = ""
                 else:
                     last_seen = revision.revision_id
-                    committer = revision.properties.get('author',
-                        revision.committer)
+                    author = revision.get_apparent_author()
 
                 if revision.revision_id not in self.revisions:
                     self.revisions[revision.revision_id] = revision
 
                 self.annomodel.append([revision.revision_id,
                                        line_no + 1,
-                                       committer,
+                                       author,
                                        revno,
                                        None,
                                        line.rstrip("\r\n")
@@ -185,7 +186,7 @@ class GAnnotateWindow(gtk.Window):
         if rev_id is None:
             return
         selected = self.revisions[rev_id]
-        self.logview.set_revision(selected)
+        self.revisionview.set_revision(selected)
         if (len(selected.parent_ids) != 0 and selected.parent_ids[0] not in
             self._no_back):
             enable_back = True
@@ -194,7 +195,7 @@ class GAnnotateWindow(gtk.Window):
         self.back_button.set_sensitive(enable_back)
 
     def _create(self):
-        self.logview = self._create_log_view()
+        self.revisionview = self._create_log_view()
         self.annoview = self._create_annotate_view()
 
         vbox = gtk.VBox(False)
@@ -221,7 +222,7 @@ class GAnnotateWindow(gtk.Window):
         
         self.pane = pane = gtk.VPaned()
         pane.add1(swbox)
-        pane.add2(self.logview)
+        pane.add2(self.revisionview)
         pane.show()
         vbox.pack_start(pane, expand=True, fill=True)
 
@@ -325,7 +326,7 @@ class GAnnotateWindow(gtk.Window):
         return tv
 
     def _create_log_view(self):
-        lv = LogView()
+        lv = RevisionView()
         lv.show()
         return lv
 
@@ -393,7 +394,6 @@ class GAnnotateWindow(gtk.Window):
                 return j - i
 
 
-
 class FakeRevision:
     """ A fake revision.
 
@@ -409,6 +409,9 @@ class FakeRevision:
         self.timezone = 0
         self.properties = {}
 
+    def get_apparent_author(self):
+        return self.committer
+
 
 class RevisionCache(object):
     """A caching revision source"""
