@@ -26,12 +26,11 @@ except:
     pass
 
 import gtk
-from dialog import question_dialog
 
 import bzrlib.errors as errors
 from bzrlib.workingtree import WorkingTree
 
-from bzrlib.plugins.gtk.dialog import error_dialog, info_dialog, warning_dialog
+from bzrlib.plugins.gtk.dialog import error_dialog, info_dialog, question_dialog, warning_dialog
 from bzrlib.plugins.gtk.errors import show_bzr_error
 from bzrlib.plugins.gtk.annotate.gannotate import GAnnotateWindow
 from bzrlib.plugins.gtk.annotate.config import GAnnotateConfig
@@ -197,7 +196,7 @@ class OliveMenu:
         branch = wt.branch
         file_id = wt.path2id(wt.relpath(os.path.join(directory, filename)))
         
-        window = GAnnotateWindow(all=False, plain=False)
+        window = GAnnotateWindow(all=False, plain=False, parent=self.app)
         window.set_title(os.path.join(directory, filename) + " - Annotate")
         config = GAnnotateConfig(window)
         window.show()
@@ -289,7 +288,11 @@ class OliveMenu:
         except NotBranchError, e:
             path = e.path
         
-        commit = CommitDialog(wt, path, not branch, self.selected)
+        if self.selected:
+            selected = os.path.join(path, self.selected)
+        else:
+            selected = None
+        commit = CommitDialog(wt=wt, selected=selected, parent=None)
         response = commit.run()
         if response != gtk.RESPONSE_NONE:
             commit.hide()
@@ -303,7 +306,7 @@ class OliveMenu:
     def diff(self, action):
         """ Right context menu -> Diff """
         wt = WorkingTree.open_containing(self.path)[0]
-        window = DiffWindow()
+        window = DiffWindow(self.app)
         parent_tree = wt.branch.repository.revision_tree(wt.branch.last_revision())
         window.set_diff(wt.branch.nick, wt, parent_tree)
         window.set_file(wt.relpath(self.path + os.sep + self.selected))
