@@ -25,10 +25,10 @@ def test_suite():
     loader = TestUtil.TestLoader()
 
     testmod_names = [
+        'test_commit',
         'test_diff',
         'test_preferences',
         'test_history',
-        'test_viz'
         ]
 
     if os.name == 'nt':
